@@ -20,8 +20,11 @@ from OpenGL.GL import GL_VERTEX_SHADER, GL_FRAGMENT_SHADER, GL_NO_ERROR, GL_COMP
 from OpenGL.GLU import gluErrorString
 from PIL import Image
 import glm
+import glfw
+
 from langutil.File import File
 from pypos3d.wftk.WFBasic import Point3d, TexCoord2f, Vector3d
+from pypos3d.wftk.GeomGroup import C_FACE_NORM, C_FACE_TEXT
 from pypos3d.wftk.WaveGeom import WFMat
 
 def printOpenGLError():
@@ -158,7 +161,8 @@ class Renderable:
     # Indicate if the GL objects have been created
     # with the FLGW windoow ID
     self.win3d = None
-
+    self.glesLevel = 0
+     
   def makeContext(self, win3d:'FLGW window'):
     self.win3d = win3d
     self.loadShader()
@@ -177,8 +181,9 @@ class Renderable:
   def loadTexture(self):
     self.texture = None
     
-  def rendering(self, win3d, MVPptr, ViewPtr, ProjectionPtr):
+  def rendering(self, win3d, MVPptr, ViewPtr, ProjectionPtr, glesLevel):
     if not self.win3d:
+      self.glesLevel = glesLevel
       self.makeContext(win3d)
   
   def DeleteBuffers(self):
@@ -223,30 +228,54 @@ class AxisGrid(Renderable):
   def loadShader(self):
     ''' Create colored Shaders. '''
     super(AxisGrid, self).loadShader()
-
-    self.shader.initShader(
-      ['''#version 300 es
-      uniform mat4 MVP;
-      layout(location = 0) in vec3 vertexPosition_modelspace;
- 
-      void main(void)
-        {
-        gl_Position = MVP *vec4(vertexPosition_modelspace,1);
-        }
-      ''',  ],\
-      ['''#version 300 es
-      // This Fragment returns a constant color
-      uniform mediump vec4 fragmentColor;
-
-      // Ouput data
-      out mediump vec4 color;
-
-      void main()
-        {
-        // Output color = color specified in the uniform constant 
-        color = fragmentColor;
-        }''', ])
-      
+    
+    if self.glesLevel==3:
+      self.shader.initShader(
+        ['''#version 300 es
+        uniform mat4 MVP;
+        layout(location = 0) in vec3 vertexPosition_modelspace;
+   
+        void main(void)
+          {
+          gl_Position = MVP * vec4(vertexPosition_modelspace,1);
+          }
+        ''',  ],\
+        ['''#version 300 es
+        // This Fragment returns a constant color
+        uniform mediump vec4 fragmentColor;
+  
+        // Ouput data
+        out mediump vec4 color;
+  
+        void main()
+          {
+          // Output color = color specified in the uniform constant 
+          color = fragmentColor;
+          }''', ])
+    else:
+      self.shader.initShader(
+        ['''#version 120
+        uniform mat4 MVP;
+        attribute vec3 vertexPosition_modelspace;
+   
+        void main(void)
+          {
+          gl_Position = MVP *vec4(vertexPosition_modelspace,1);
+          }
+        ''',  ],\
+        ['''#version 120
+        // This Fragment returns a constant color
+        uniform vec4 fragmentColor;
+  
+        // Ouput data
+        varying vec4 color;
+  
+        void main()
+          {
+          // Output color = color specified in the uniform constant 
+          gl_FragColor = fragmentColor;
+          }''', ])
+  
     self.MVP_ID = glGetUniformLocation(self.shader.program,"MVP")
     self.shader.Color_ID = glGetUniformLocation(self.shader.program, "fragmentColor")
 
@@ -279,8 +308,8 @@ class AxisGrid(Renderable):
     else:
       self.gridBuffer = None
       
-  def rendering(self, win3d, MVP, View, Projection):
-    super(AxisGrid, self).rendering(win3d, MVP, View, Projection)
+  def rendering(self, win3d, MVP, View, Projection, glesLevel):
+    super(AxisGrid, self).rendering(win3d, MVP, View, Projection, glesLevel)
 
     self.shader.begin()
     
@@ -439,14 +468,60 @@ class RendWaveGeom(Renderable):
    
     # Create Textured Shaders 
     self.shader = Shader()
-    self.shader.initShader( \
-      ['''#version 300 es
+    if self.glesLevel==3:
+      self.shader.initShader( \
+        ['''#version 300 es
+        // Input vertex data, different for all executions of this shader.
+        layout(location = 0) in vec3 vertexPosition_modelspace;
+        layout(location = 1) in vec2 vertexUV;
+  
+        // Output data ; will be interpolated for each fragment.
+        out vec2 UV;
+  
+        // Values that stay constant for the whole mesh.
+        uniform mat4 MVP;
+        uniform vec3 OFFSET;
+        
+        void main()
+          {
+          vec3 position_offset = vertexPosition_modelspace.xyz + OFFSET.xyz;
+          // Output position of the vertex, in clip space : MVP * position
+          gl_Position =  MVP * vec4(position_offset,1);
+    
+          // UV of the vertex. No special space for this one.
+          UV = vertexUV;
+          }
+        ''', ], \
+        ['''#version 300 es
+        // Interpolated values from the vertex shaders
+        in mediump vec2 UV;
+        
+        // Diffuse color to combine (blending --> multiply)
+        uniform mediump vec4 fragmentColor;
+        
+        // Ouput data
+        out mediump vec4 color;
+  
+        // Values that stay constant for the whole mesh.
+        uniform sampler2D myTextureSampler;
+  
+        void main()
+          {
+          // Output color = color of the texture at the specified UV
+          // I want to preserve the alpha channel of the underlying color
+          color.rgb = texture( myTextureSampler, UV ).rgb * fragmentColor.rgb;
+          color.a = fragmentColor.a;        
+          }      
+        ''', ])
+    else:
+      self.shader.initShader(
+        ['''#version 120
       // Input vertex data, different for all executions of this shader.
-      layout(location = 0) in vec3 vertexPosition_modelspace;
-      layout(location = 1) in vec2 vertexUV;
+      attribute vec3 vertexPosition_modelspace;
+      attribute vec2 vertexUV;
 
       // Output data ; will be interpolated for each fragment.
-      out vec2 UV;
+      varying vec2 UV;
 
       // Values that stay constant for the whole mesh.
       uniform mat4 MVP;
@@ -462,15 +537,14 @@ class RendWaveGeom(Renderable):
         UV = vertexUV;
         }
       ''', ], \
-      ['''#version 300 es
+      ['''#version 120
       // Interpolated values from the vertex shaders
-      in mediump vec2 UV;
+      varying vec2 UV;
       
       // Diffuse color to combine (blending --> multiply)
-      uniform mediump vec4 fragmentColor;
+      uniform vec4 fragmentColor;
       
-      // Ouput data
-      out mediump vec4 color;
+      // Ouput data vec4 color;
 
       // Values that stay constant for the whole mesh.
       uniform sampler2D myTextureSampler;
@@ -479,10 +553,11 @@ class RendWaveGeom(Renderable):
         {
         // Output color = color of the texture at the specified UV
         // I want to preserve the alpha channel of the underlying color
-        color.rgb = texture( myTextureSampler, UV ).rgb * fragmentColor.rgb;
-        color.a = fragmentColor.a;        
+        gl_FragColor.rgb = texture2D( myTextureSampler, UV ).rgb * fragmentColor.rgb;
+        gl_FragColor.a = fragmentColor.a;        
         }      
       ''', ])
+      
 
 
     self.shader.MVP_ID = glGetUniformLocation(self.shader.program, "MVP")
@@ -492,11 +567,11 @@ class RendWaveGeom(Renderable):
     
     # Create Colored Shaders 
     self.shaderC = Shader()
-    self.shaderC.initShader( \
+    if self.glesLevel==3:
+      self.shaderC.initShader( \
       [ '''#version 300 es
       // Input vertex data, different for all executions of this shader.
       layout(location = 0) in vec3 vertexPosition_modelspace;
-      //layout(location = 1) in vec3 vertexColor;
 
       // Output data ; will be interpolated for each fragment.
       out mediump vec3 fragmentColor;
@@ -507,9 +582,6 @@ class RendWaveGeom(Renderable):
 
       void main()
         {  
-        // Output position of the vertex, in clip space : MVP * position
-        // gl_Position =  MVP * vec4(vertexPosition_modelspace,1);
-        
         vec3 position_offset = vertexPosition_modelspace.xyz + OFFSET.xyz;
         // Output position of the vertex, in clip space : MVP * position
         gl_Position =  MVP * vec4(position_offset,1);        
@@ -528,6 +600,40 @@ class RendWaveGeom(Renderable):
         color = fragmentColor;
         }
        ''', ])
+    else:
+      self.shaderC.initShader( \
+      [ '''#version 120
+      // Input vertex data, different for all executions of this shader.
+      attribute vec3 vertexPosition_modelspace;
+
+      // Output data ; will be interpolated for each fragment.
+      varying vec3 fragmentColor;
+      
+      // Values that stay constant for the whole mesh.
+      uniform mat4 MVP;
+      uniform vec3 OFFSET;
+
+      void main()
+        {  
+        vec3 position_offset = vertexPosition_modelspace.xyz + OFFSET.xyz;
+        // Output position of the vertex, in clip space : MVP * position
+        gl_Position =  MVP * vec4(position_offset,1);        
+        }
+       ''', ],
+      ['''#version 120
+      // This Fragment returns a constant color
+      uniform vec4 fragmentColor;
+
+      // Ouput data
+      varying vec4 color;
+
+      void main()
+        {
+        // Output color = color specified in the uniform constant 
+        gl_FragColor = fragmentColor;
+        }
+       ''', ])
+      
     
     self.shaderC.MVP_ID = glGetUniformLocation(self.shaderC.program, "MVP")
     self.shaderC.OFFSET_ID = glGetUniformLocation(self.shaderC.program, "OFFSET")
@@ -554,30 +660,36 @@ class RendWaveGeom(Renderable):
 
     # Write the face of each group
     for grp in geom.getGroups():
+      
+      defaultTbl = [-1] * (4+max( grp.stripCount[noface + 1]-grp.stripCount[noface] for noface in range(grp.getNbFace()) ))
+      
       vertIdxTbl = grp.vertIdx
-      tvertIdxTbl = grp.tvertIdx if (self.hasTexture and grp.tvertIdx) else [-1]*len(vertIdxTbl)
-      normIdxTbl = grp.normIdx if (self.hasNormal and grp.normIdx) else [-1]*len(vertIdxTbl)
+      #tvertIdxTbl = grp.tvertIdx if (self.hasTexture and grp.tvertIdx) else [-1]*len(vertIdxTbl)
+      #normIdxTbl = grp.normIdx if (self.hasNormal and grp.normIdx) else [-1]*len(vertIdxTbl)
       
       grpModels = [ RendGroupMat(grp, self.libMat.get(geom.lstMat[i]), self.hasTexture, self.hasNormal, coords, texs, norms) \
                     for i in range(nbMat) ] if self.libMat else \
                   [ RendGroupMat(grp, None, self.hasTexture, self.hasNormal, coords, texs, norms) for i in range(nbMat) ]
      
-      for noface in range(0, grp.getNbFace()):
+      for noface, fao in enumerate(grp.matIdx):
         startIdx = grp.getFaceStartIdx(noface)
         lastIdx = grp.getFaceLastIdx(noface)
         argc = lastIdx - startIdx
         matIdx = grp.getMatIdx(noface)
         glcont = grpModels[matIdx]
-
+        
         v0 = vertIdxTbl[startIdx]
         v1 = vertIdxTbl[startIdx+1]
         v2 = vertIdxTbl[startIdx+2]
+        
+        tvertIdxTbl = fao.tvertIdx if fao.fAttr & C_FACE_TEXT else defaultTbl
+        normIdxTbl = fao.normIdx if fao.fAttr & C_FACE_NORM else defaultTbl
 
         if argc==3:
           # self.add_face(v0, v1, v2, isprotected, matIdx, grpNo)
-          glcont.addGLPoint(v0, tvertIdxTbl[startIdx], normIdxTbl[startIdx])
-          glcont.addGLPoint(v1, tvertIdxTbl[startIdx+1], normIdxTbl[startIdx+1])
-          glcont.addGLPoint(v2, tvertIdxTbl[startIdx+2], normIdxTbl[startIdx+2])
+          glcont.addGLPoint(v0, tvertIdxTbl[0], normIdxTbl[0])
+          glcont.addGLPoint(v1, tvertIdxTbl[0+1], normIdxTbl[0+1])
+          glcont.addGLPoint(v2, tvertIdxTbl[0+2], normIdxTbl[0+2])
           
         elif argc==4:
           v3 = vertIdxTbl[startIdx+3]          
@@ -591,23 +703,22 @@ class RendWaveGeom(Renderable):
           a_13 = (1 - e1.dot(e0)) + (1.0 - e3.dot(e2))
           # Comparison with Java Algo : Accuracy changes
           if a_02 <= a_13:
-            glcont.addGLPoint(v0, tvertIdxTbl[startIdx], normIdxTbl[startIdx])
-            glcont.addGLPoint(v1, tvertIdxTbl[startIdx+1], normIdxTbl[startIdx+1])
-            glcont.addGLPoint(v2, tvertIdxTbl[startIdx+2], normIdxTbl[startIdx+2])
+            glcont.addGLPoint(v0, tvertIdxTbl[0], normIdxTbl[0])
+            glcont.addGLPoint(v1, tvertIdxTbl[0+1], normIdxTbl[0+1])
+            glcont.addGLPoint(v2, tvertIdxTbl[0+2], normIdxTbl[0+2])
 
-            glcont.addGLPoint(v0, tvertIdxTbl[startIdx], normIdxTbl[startIdx])
-            glcont.addGLPoint(v2, tvertIdxTbl[startIdx+2], normIdxTbl[startIdx+2])
-            glcont.addGLPoint(v3, tvertIdxTbl[startIdx+3], normIdxTbl[startIdx+3])
+            glcont.addGLPoint(v0, tvertIdxTbl[0], normIdxTbl[0])
+            glcont.addGLPoint(v2, tvertIdxTbl[0+2], normIdxTbl[0+2])
+            glcont.addGLPoint(v3, tvertIdxTbl[0+3], normIdxTbl[0+3])
             
           else:
-            glcont.addGLPoint(v0, tvertIdxTbl[startIdx], normIdxTbl[startIdx])
-            glcont.addGLPoint(v1, tvertIdxTbl[startIdx+1], normIdxTbl[startIdx+1])
-            glcont.addGLPoint(v3, tvertIdxTbl[startIdx+3], normIdxTbl[startIdx+3])
+            glcont.addGLPoint(v0, tvertIdxTbl[0], normIdxTbl[0])
+            glcont.addGLPoint(v1, tvertIdxTbl[0+1], normIdxTbl[0+1])
+            glcont.addGLPoint(v3, tvertIdxTbl[0+3], normIdxTbl[0+3])
 
-            glcont.addGLPoint(v1, tvertIdxTbl[startIdx+1], normIdxTbl[startIdx+1])
-            glcont.addGLPoint(v2, tvertIdxTbl[startIdx+2], normIdxTbl[startIdx+2])
-            glcont.addGLPoint(v3, tvertIdxTbl[startIdx+3], normIdxTbl[startIdx+3])
-            
+            glcont.addGLPoint(v1, tvertIdxTbl[0+1], normIdxTbl[0+1])
+            glcont.addGLPoint(v2, tvertIdxTbl[0+2], normIdxTbl[0+2])
+            glcont.addGLPoint(v3, tvertIdxTbl[0+3], normIdxTbl[0+3])
         else:
           # Input polygon many sides (>4). Triangularize it.
           # Compute the isobarycentre
@@ -616,27 +727,27 @@ class RendWaveGeom(Renderable):
           n = Vector3d()
           for i,vno in enumerate(vertIdxTbl[startIdx:lastIdx]):
             isoG.add(geom.coordList[vno])
-            if self.hasNormal:
-              n.add(geom.normList[grp.normIdx[i+startIdx]])
+            if fao.fAttr & C_FACE_NORM:
+              n.add(geom.normList[normIdxTbl[i]])
                 
-            if self.hasTexture:
-              t.add(geom.texList[tvertIdxTbl[i+startIdx]])
+            if fao.fAttr & C_FACE_TEXT:
+              t.add(geom.texList[tvertIdxTbl[i]])
               
           factor = 1.0/float(argc)                           
           isoG.scale(factor)
           t.scale(factor)
           n.scale(factor)
           
-          for i in range(startIdx, lastIdx-1):
+          for j,i in enumerate(range(startIdx, lastIdx-1)):
             #self.add_face(gno, , , isprotected, matIdx, grpNo)
             glcont.addExtraGLPoint((isoG.x, isoG.y, isoG.z), (t.x, t.y), (n.x, n.y, n.z))
-            glcont.addGLPoint(vertIdxTbl[i], tvertIdxTbl[i], normIdxTbl[i])
-            glcont.addGLPoint(vertIdxTbl[i+1], tvertIdxTbl[i+1], normIdxTbl[i+1])
+            glcont.addGLPoint(vertIdxTbl[i], tvertIdxTbl[j], normIdxTbl[j])
+            glcont.addGLPoint(vertIdxTbl[i+1], tvertIdxTbl[j+1], normIdxTbl[j+1])
 
           #self.add_face(gno, vertIdxTbl[lastIdx-1], vertIdxTbl[startIdx], isprotected, matIdx, grpNo)
           glcont.addExtraGLPoint((isoG.x, isoG.y, isoG.z), (t.x, t.y), (n.x, n.y, n.z))
-          glcont.addGLPoint(vertIdxTbl[lastIdx-1], tvertIdxTbl[lastIdx-1], normIdxTbl[lastIdx-1])
-          glcont.addGLPoint(vertIdxTbl[startIdx], tvertIdxTbl[startIdx], normIdxTbl[startIdx])
+          glcont.addGLPoint(vertIdxTbl[lastIdx-1], tvertIdxTbl[-1], normIdxTbl[-1])
+          glcont.addGLPoint(vertIdxTbl[startIdx], tvertIdxTbl[0], normIdxTbl[0])
 
       # Finish the Renderable group creation
       nbtri += len(glcont.indices)
@@ -675,8 +786,8 @@ class RendWaveGeom(Renderable):
           glBufferData(GL_ELEMENT_ARRAY_BUFFER, len(glcont.texcoords)*4, (GLfloat * len(glcont.texcoords))(*glcont.texcoords), GL_STATIC_DRAW)
           #else: # In case of error (load error), go back to color mode
 
-  def rendering(self, win3d, MVP, View, Projection):
-    super(RendWaveGeom, self).rendering(win3d, MVP, View, Projection)
+  def rendering(self, win3d, MVP, View, Projection, glesLevel):
+    super(RendWaveGeom, self).rendering(win3d, MVP, View, Projection, glesLevel)
     
     for glcont in [ d for grpDict in self._models for d in grpDict.values()  ]:
       
