@@ -6,6 +6,7 @@ import threading
 import multiprocessing as mp
 
 from OpenGL.GL import glDepthFunc, glEnable, glDisable, glBlendFunc, glGetString, glClear, glViewport, glPolygonMode
+
 from OpenGL.GL import GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_RENDERER, GL_SHADING_LANGUAGE_VERSION, \
                       GL_COLOR_BUFFER_BIT, GL_DEPTH_BUFFER_BIT, GL_LINE, GL_FRONT_AND_BACK, GL_FILL, GL_CULL_FACE, \
                       GL_LESS, GL_DEPTH_TEST, GL_BLEND
@@ -75,6 +76,9 @@ class View3dWindow:
 
   ''' Indicates if the MainLoop runs '''
   __IN_MAINLOOP = False
+
+  ''' GLES level to use for shaders '''
+  __GLES_LEVEL = 3
   
   
   @classmethod
@@ -88,6 +92,8 @@ class View3dWindow:
     
   @classmethod
   def start(cls):
+    ''' Initialize the GLFW library. '''
+    
     View3dWindow.__W3DLOCK.acquire()
     if not View3dWindow.__GLFW_STARTED:      
       # Initialize glfw
@@ -259,6 +265,7 @@ class View3dWindow:
   def __createInternal(self):
     ''' Internal GLFW and OpenGL creation of the window. 
     Should be called from the MainLoop (GLFW initialized)
+    Also check for GLES supported extension
     '''
     
     # Create a GLFW window
@@ -277,9 +284,25 @@ class View3dWindow:
 
     # Initialize GL and create default objects (Axis, Grids, ...)
     self.ogl_init()
+    
+    if glfw.extension_supported('GL_ARB_ES3_compatibility'):
+      logging.info('Extension supported:%s', 'GL_ARB_ES3_compatibility')
+      View3dWindow.__GLES_LEVEL = 3      
+    else:       
+      logging.info('Using GLES 120 Extension')
+      View3dWindow.__GLES_LEVEL = 1
+
     logging.info('Window[{:s}] {:d}x{:d} created - GL Context:Renderer:{:s} - Shading Language:{:s}'.format(self.name, self.width, self.height, \
                  str(glGetString(GL_RENDERER)), str(glGetString(GL_SHADING_LANGUAGE_VERSION))))
     
+    
+    # List Available Extensions
+    #     NumberOfExtensions = glGetIntegerv(GL_NUM_EXTENSIONS)
+    #     for i in range(0, NumberOfExtensions):
+    #       ccc = glGetStringi(GL_EXTENSIONS, i)
+    #       print(str(ccc))
+    
+    
   
   def __init__(self, width=800, height=480, connIn=None, name='pypos3dv.viewer', axisGrid=None, \
                background=(0.1, 0.1, 0.1, 0.8), usemtlfiles=True, defaultmtl=None, imgdirpath='.'):
@@ -489,7 +512,7 @@ class View3dWindow:
     # Draw visible meshes (Renderable)
     for mesh in self.meshes.values():
       if mesh.isVisible:        
-        mesh.rendering(self.win, self._MVP, self.ViewMatrix, self.ProjectionMatrix)
+        mesh.rendering(self.win, self._MVP, self.ViewMatrix, self.ProjectionMatrix, View3dWindow.__GLES_LEVEL)
           
   def moveFoward(self, forward):
     self.obs += self.direction*forward
